@@ -1,4 +1,4 @@
-/* Copyright 2007 Matthew Barnes
+/* Copyright 2007, 2008 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -135,55 +135,19 @@ audit_read (GvaProcess *process,
             GvaAuditData *data)
 {
         gchar *line;
-        const gchar *name;
-        const gchar *status = NULL;
-        const gchar *token;
+        gchar *name;
+        gchar *status;
         GString *string;
         gpointer value;
 
-        /* Output for -verifyroms is as follows:
-         *
-         * romset puckman is good
-         * romset puckmana [puckman] is good
-         * romset puckmanf [puckman] is good
-         * ...
-         * digdug  : 51xx.bin (1024 bytes) - NOT FOUND
-         * digdug  : 53xx.bin (1024 bytes) - NOT FOUND
-         * romset digdug is bad
-         * ...
-         *
-         * - Status may be "good", "bad", or "best available".
-         *
-         * - Older MAMEs used "correct" and "incorrect" instead of
-         *   "good" and "bad".  Convert to the newer form if seen.
-         *
-         * - Similar output for -verifysamples.
-         */
-
         line = g_strchomp (gva_process_stdout_read_line (process));
 
         value = GUINT_TO_POINTER (data->output->len);
         g_ptr_array_add (data->output, g_strdup (line));
 
-        if ((token = strtok (line, " ")) == NULL)
+        if (!gva_mame_verify_parse (line, &name, &status))
                 goto exit;
 
-        if (strcmp (token, data->column) != 0)
-                goto exit;
-
-        name = strtok (NULL, " ");
-        while ((token = strtok (NULL, " ")) != NULL)
-                status = token;
-
-        if (name == NULL || status == NULL)
-                goto exit;
-
-        /* Normalize the status. */
-        if (strcmp (status, "correct") == 0)
-                status = "good";
-        else if (strcmp (status, "incorrect") == 0)
-                status = "bad";
-
         g_hash_table_insert (data->output_index, g_strdup (name), value);
 
         /* Build a quoted, comma-separated list of games. */
@@ -202,6 +166,9 @@ audit_read (GvaProcess *process,
 
         gva_process_inc_progress (process);
 
+        g_free (name);
+        g_free (status);
+
 exit:
         g_free (line);
 }
@@ -228,6 +195,7 @@ audit_exit (GvaProcess *process,
             gint status,
             GvaAuditData *data)
 {
+        gchar *sql;
         GError *error = NULL;
 
         if (process->error != NULL)
@@ -236,6 +204,11 @@ audit_exit (GvaProcess *process,
         gva_db_transaction_begin (&error);
         gva_error_handle (&error);
 
+        sql = g_strdup_printf ("UPDATE game SET %s=NULL", data->column);
+        gva_db_execute (sql, &error);
+        gva_error_handle (&error);
+        g_free (sql);
+
         g_hash_table_foreach (
                 data->status_index, (GHFunc) audit_exit_foreach, data);
 
@@ -285,6 +258,7 @@ audit_run_save_dialog (void)
 
         gtk_file_chooser_set_current_folder (file_chooser, folder);
         gtk_file_chooser_set_current_name (file_chooser, name);
+        gtk_file_chooser_set_do_overwrite_confirmation (file_chooser, TRUE);
 
         g_free (name);
         g_free (folder);
@@ -365,7 +339,7 @@ gva_audit_roms (GError **error)
         GvaProcess *process;
         GvaAuditData *data;
 
-        process = gva_mame_verify_roms (error);
+        process = gva_mame_verify_all_roms (error);
         if (process == NULL)
                 return NULL;
 
@@ -408,7 +382,7 @@ gva_audit_samples (GError **error)
         GvaProcess *process;
         GvaAuditData *data;
 
-        process = gva_mame_verify_roms (error);
+        process = gva_mame_verify_all_samples (error);
         if (process == NULL)
                 return NULL;
 
@@ -495,3 +469,57 @@ gva_audit_save_errors (void)
         g_free (mame_version);
         g_string_free (contents, TRUE);
 }
+
+/**
+ * gva_audit_detect_changes:
+ *
+ * Attempts to detect ROM and sample file changes since the last program
+ * execution by scanning the timestamps on the directories listed in the
+ * "rompath" and "samplepath" configuration values.  It returns %TRUE if
+ * any of those timestamps are more recent than the game database.
+ *
+ * Returns: %TRUE if changes were detected
+ **/
+gboolean
+gva_audit_detect_changes (void)
+{
+        gboolean changes = FALSE;
+        gchar **directories;
+        time_t db_mtime;
+        struct stat st;
+        guint length, ii;
+        GError *error = NULL;
+
+        if (g_stat (gva_db_get_filename (), &st) < 0)
+                return FALSE;
+
+        db_mtime = st.st_mtime;
+
+        directories = gva_mame_get_search_paths ("rompath", &error);
+        length = (directories != NULL) ? g_strv_length (directories) : 0;
+        gva_error_handle (&error);
+
+        for (ii = 0; ii < length; ii++)
+                if (g_stat (directories[ii], &st) == 0)
+                {
+                        changes |= (db_mtime < st.st_mtime);
+                        changes |= (db_mtime < st.st_ctime);
+                }
+
+        g_strfreev (directories);
+
+        directories = gva_mame_get_search_paths ("samplepath", &error);
+        length = (directories != NULL) ? g_strv_length (directories) : 0;
+        gva_error_handle (&error);
+
+        for (ii = 0; ii < length; ii++)
+                if (g_stat (directories[ii], &st) == 0)
+                {
+                        changes |= (db_mtime < st.st_mtime);
+                        changes |= (db_mtime < st.st_ctime);
+                }
+
+        g_strfreev (directories);
+
+        return changes;
+}
