@@ -1,4 +1,4 @@
-/* Copyright 2007 Matthew Barnes
+/* Copyright 2007, 2008 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -50,11 +50,10 @@
 
 G_BEGIN_DECLS
 
-typedef enum _GvaGameStoreColumn GvaGameStoreColumn;
 typedef struct _GvaGameStore GvaGameStore;
 typedef struct _GvaGameStoreClass GvaGameStoreClass;
 
-enum _GvaGameStoreColumn
+typedef enum
 {
         GVA_GAME_STORE_COLUMN_NAME,               /* G_TYPE_STRING */
         GVA_GAME_STORE_COLUMN_CATEGORY,           /* G_TYPE_STRING */
@@ -84,10 +83,12 @@ enum _GvaGameStoreColumn
         GVA_GAME_STORE_COLUMN_DRIVER_PROTECTION,  /* G_TYPE_STRING */
         GVA_GAME_STORE_COLUMN_DRIVER_SAVESTATE,   /* G_TYPE_STRING */
         GVA_GAME_STORE_COLUMN_DRIVER_PALETTESIZE, /* G_TYPE_INT */
+        GVA_GAME_STORE_COLUMN_COMMENT,            /* G_TYPE_STRING */
+        GVA_GAME_STORE_COLUMN_INODE,              /* G_TYPE_INT64 */
         GVA_GAME_STORE_COLUMN_INPFILE,            /* G_TYPE_STRING */
         GVA_GAME_STORE_COLUMN_TIME,               /* GVA_TYPE_TIME */
         GVA_GAME_STORE_NUM_COLUMNS
-};
+} GvaGameStoreColumn;
 
 /**
  * GvaGameStore:
