@@ -1,4 +1,4 @@
-/* Copyright 2007 Matthew Barnes
+/* Copyright 2007, 2008 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -49,7 +49,8 @@ static const gchar *authors[] =
         "Matthew Barnes <mbarnes@redhat.com>",
         NULL
 };
-static const gchar *copyright = "Copyright \xC2\xA9 2007 Matthew Barnes";
+static const gchar *copyright =
+"Copyright \xC2\xA9 2007, 2008 Matthew Barnes";
 static const gchar *license =
 "GNOME Video Arcade is free software; you can redistribute it "
 "and/or modify it under the terms of the GNU General Public License "
@@ -291,7 +292,7 @@ action_play_back_cb (GtkAction *action)
 /**
  * GVA_ACTION_PREFERENCES:
  *
- * Activation of this action makes the "Preferences" window visible.
+ * Activation of this action presents the "Preferences" window.
  *
  * Main menu item: Edit -> Preferences
  **/
@@ -338,7 +339,7 @@ action_previous_game_cb (GtkAction *action)
 /**
  * GVA_ACTION_PROPERTIES:
  *
- * Activation of this action makes the "Properties" window visible.
+ * Activation of this action presents the "Properties" window.
  *
  * Main menu item: View -> Properties
  **/
@@ -443,7 +444,7 @@ action_save_errors_cb (GtkAction *action)
 /**
  * GVA_ACTION_SEARCH:
  *
- * Activation of this action makes the "Search for Games" window visible.
+ * Activation of this action presents the "Search for Games" window.
  *
  * Main menu item: Edit -> Search...
  **/
@@ -456,7 +457,7 @@ action_search_cb (GtkAction *action)
 /**
  * GVA_ACTION_SHOW_PLAY_BACK:
  *
- * Activation of this action makes the "Recorded Games" window visible.
+ * Activation of this action presents the "Recorded Games" window.
  *
  * Main menu item: Game -> Play Back...
  **/
@@ -541,7 +542,7 @@ static GtkActionEntry entries[] =
           GTK_STOCK_HELP,
           N_("_Contents"),
           NULL,
-          NULL,
+          N_("Open the help documentation"),
           G_CALLBACK (action_contents_cb) },
 
         { "insert-favorite",
