@@ -1,4 +1,4 @@
-/* Copyright 2007 Matthew Barnes
+/* Copyright 2007, 2008 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -81,16 +81,40 @@ start (void)
         }
 
         if (gva_db_needs_rebuilt ())
+        {
+                gboolean success = TRUE;
+
+                gva_main_progress_bar_show ();
+
+                success &= gva_main_build_database (&error);
+                gva_error_handle (&error);
+
+                gva_main_progress_bar_set_fraction (0.0);
+
+                success &= gva_main_analyze_roms (&error);
+                gva_error_handle (&error);
+
+                gva_main_progress_bar_hide ();
+
+                if (!success)
+                        return;
+        }
+        else if (gva_audit_detect_changes ())
         {
                 gboolean success;
 
-                success = gva_main_build_database (&error);
+                gva_main_progress_bar_show ();
+
+                success = gva_main_analyze_roms (&error);
                 gva_error_handle (&error);
 
+                gva_main_progress_bar_hide ();
+
                 if (!success)
                         return;
         }
 
+        gtk_action_set_sensitive (GVA_ACTION_SEARCH, TRUE);
         gtk_action_set_sensitive (GVA_ACTION_VIEW_AVAILABLE, TRUE);
         gtk_action_set_sensitive (GVA_ACTION_VIEW_FAVORITES, TRUE);
         gtk_action_set_sensitive (GVA_ACTION_VIEW_RESULTS, TRUE);
